@@ -7,9 +7,6 @@
 #' @param object PENSE regularization path to extract residuals from.
 #' @param newdata an optional matrix of new predictor values.
 #'                If missing, the fitted values are computed.
-#' @param exact defunct Always gives a warning if `lambda` is not part of the fitted
-#'    sequence and coefficients need to be interpolated.
-#' @param correction defunct.
 #' @param ... currently not used.
 #' @return a numeric vector of residuals for the given penalization level.
 #'
@@ -23,15 +20,7 @@
 #' @importFrom rlang abort
 #' @importFrom stats coef
 #'
-predict.pense_fit <- function(object, newdata, alpha = NULL, lambda,
-                              exact = deprecated(), correction = deprecated(), ...) {
-  if (is_present(exact)) {
-    deprecate_stop('2.0.0', 'residuals(exact=)')
-  }
-  if (is_present(correction)) {
-    deprecate_stop('2.0.0', 'residuals(correction=)')
-  }
-
+predict.pense_fit <- function(object, newdata, alpha = NULL, lambda, ...) {
   coef_est <- coef(object, alpha = alpha, lambda = lambda, concat = FALSE)
 
   if (missing(newdata)) {
@@ -58,9 +47,6 @@ predict.pense_fit <- function(object, newdata, alpha = NULL, lambda,
 #' @param object PENSE with cross-validated hyper-parameters to extract coefficients from.
 #' @param newdata an optional matrix of new predictor values.
 #'                If missing, the fitted values are computed.
-#' @param exact deprecated. Always gives a warning if `lambda` is not part of the
-#'    fitted sequence and coefficients are interpolated.
-#' @param correction defunct.
 #' @param ... currently not used.
 #' @return a numeric vector of residuals for the given penalization level.
 #'
@@ -73,15 +59,7 @@ predict.pense_fit <- function(object, newdata, alpha = NULL, lambda,
 #' @importFrom lifecycle deprecate_stop deprecated is_present
 #' @importFrom rlang warn
 #' @importFrom stats coef
-predict.pense_cvfit <- function(object, newdata, alpha = NULL, lambda = 'min', se_mult = 1,
-                                exact = deprecated(), correction = deprecated(), ...) {
-  if (is_present(exact)) {
-    deprecate_stop('2.0.0', 'coef(exact=)')
-  }
-  if (is_present(correction)) {
-    deprecate_stop('2.0.0', 'coef(correction=)')
-  }
-
+predict.pense_cvfit <- function(object, newdata, alpha = NULL, lambda = 'min', se_mult = 1, ...) {
   coef_est <- coef(object, alpha = alpha, lambda = lambda, se_mult = se_mult, concat = FALSE)
 
   if (missing(newdata)) {
@@ -106,9 +84,6 @@ predict.pense_cvfit <- function(object, newdata, alpha = NULL, lambda = 'min', s
 #'
 #' @template hyper_param-fit
 #' @param object PENSE regularization path to extract residuals from.
-#' @param exact defunct Always gives a warning if `lambda` is not part of
-#'    the fitted sequence and coefficients need to be interpolated.
-#' @param correction defunct.
 #' @param ... currently not used.
 #' @return a numeric vector of residuals for the given penalization level.
 #'
@@ -116,8 +91,7 @@ predict.pense_cvfit <- function(object, newdata, alpha = NULL, lambda = 'min', s
 #'
 #' @example examples/residuals.R
 #' @export
-residuals.pense_fit <- function(object, alpha = NULL, lambda, exact = deprecated(),
-                                correction = deprecated(), ...) {
+residuals.pense_fit <- function(object, alpha = NULL, lambda, ...) {
   train_y <- eval.parent(object$call$y)
   cl <- match.call(expand.dots = FALSE)
   cl[[1L]] <- quote(predict)
@@ -132,9 +106,6 @@ residuals.pense_fit <- function(object, alpha = NULL, lambda, exact = deprecated
 #' @template hyper_param-cv
 #' @param object PENSE with cross-validated hyper-parameters to extract coefficients from.
 #' @param se_mult If `lambda = "se"`, the multiple of standard errors to tolerate.
-#' @param exact deprecated. Always gives a warning if `lambda` is not part of the fitted sequence and coefficients
-#'    are interpolated.
-#' @param correction defunct.
 #' @param ... currently not used.
 #' @return a numeric vector of residuals for the given penalization level.
 #'
@@ -142,8 +113,7 @@ residuals.pense_fit <- function(object, alpha = NULL, lambda, exact = deprecated
 #'
 #' @example examples/residuals.R
 #' @export
-residuals.pense_cvfit <- function(object, alpha = NULL, lambda = 'min', se_mult = 1,
-                                  exact = deprecated(), correction = deprecated(), ...) {
+residuals.pense_cvfit <- function(object, alpha = NULL, lambda = 'min', se_mult = 1, ...) {
   train_y <- eval.parent(object$call$y)
   cl <- match.call(expand.dots = FALSE)
   cl[[1L]] <- quote(predict)
